#!/bin/bash

touch ../lib/converter.conf

bzip2 -d xubetrieb_000.xml.bz2
mv xubetrieb_000.xml ../data/

bzip2 -d xubetrieb_001_emof_whole.xml.bz2
mv xubetrieb_001_emof_whole.xml ../data/

echo "Done."
